#!/bin/bash
# adriane-common.sh
# Global and local configuration, macros for ADRIANE
# (C) Klaus Knopper 2007

# Global variables
PATH="/bin:/sbin:/usr/bin:/usr/sbin:/usr/X11R6/bin:/usr/local/bin:/usr/local/sbin:$HOME/bin"
export PATH

export ADRIANE="1.2"

NOOK=""
DEFAULTWIDTH=""
DEFAULTHEIGHT=""
DIALOG="dialog"
# ADRIANE under X
if [ -n "$DISPLAY" ] && type -t Xdialog >/dev/null 2>&1; then
 DIALOG="Xdialog 1>&2 2>/dev/null --stdout"
 export XDIALOG_HIGH_DIALOG_COMPAT=1
 export XDIALOG_NO_GMSGS=1
 DEFAULTWIDTH=75
 DEFAULTHEIGHT=18
else
 # Quick check for support of "--no-ok" in dialog
 case "$($DIALOG --help 2>&1)" in *--no-ok*) NOOK="--no-ok";; esac
 export DIALOGOPTS="--begin 0 0 --no-shadow"
 [ -f /etc/adriane/dialogrc ] && export DIALOGRC="/etc/adriane/dialogrc"
fi

# Defaults, overwritten by /etc/adriane/adriane.conf and local config
COLUMNS="80"
LINES="25"
# Default text editor
EDITOR="nano -t"
VIEWER="nano -t -v"
ESPEAK_PITCH="80"
ESPEAK_SPEED="100"
COMMAND_WWW="elinks"
COMMAND_EMAIL="mutt"
COMMAND_CHAT="irssi"
COMMAND_SPEAK="spd-say"

# Global configuration, language
if [ -r "/etc/sysconfig/i18n" ]; then
 . "/etc/sysconfig/i18n"
elif [ -r "/etc/default/locale" ]; then
 . "/etc/default/locale"
fi

# Source common functions & global variables
# (/etc/adriane/adriane.conf, $HOME/.adriane/adriane.conf)

# Global ADRIANE configuration
[ -r "/etc/adriane/adriane.conf" ] && . "/etc/adriane/adriane.conf"

# User configuration
[ -r "$HOME/.adriane/adriane.conf" ] && . "$HOME/.adriane/adriane.conf"

# TODO: Bugfix: Xdialog problems with UTF-8?

TMP="/tmp/adriane.tmp.$$"

# Localization is a little more complicated here,
# since adriane-common is included by many files.
MSG_HELP=$(gettext adriane-common "Help")
MSG_ERROR_HELP=$(gettext adriane-common "Help text not available yet.")
MSG_NO=$(gettext adriane-common "No")
MSG_YES=$(gettext adriane-common "Yes")
MSG_FULLMESSAGE=$(gettext adriane-common "Show full message")
MSG_ERROR=$(gettext adriane-common "Error")
MSG_SELECT=$(gettext adriane-common "Select")
MSG_QUIT=$(gettext adriane-common "Quit")
MSG_BACK=$(gettext adriane-common "Back")
MSG_EDIT=$(gettext adriane-common "Edit")
MSG_MANAGE_FILES=$(gettext adriane-common "Manage saved files")
MSG_CANCEL=$(gettext adriane-common "Cancel")
MSG_DIRECTORY=$(gettext adriane-common "Directory")

# Functions

# Delete these files with bailout when finished. Can be extended later.
TMPFILES=( "$TMP" "$TMP.err" "$TMP.pass" "$TMP.list" "$TMP.msg" "$TMP.pnm" "$TMP.txt" "$TMP.progress" "$TMP.name" "$TMP.tmp" "$TMP.common" "$TMP.yesno" )

# Add file to delete list if not already present
addtmpfile(){
 local file
 for file in "$@"; do
  local i
  for ((i=0;i<${#TMPFILES[@]}]; i++)); do
   case "$file" in "${TMPFILES[$i]}]") continue 2;; esac
  done
  TMPFILES[${#TMPFILES[@]}]="$file"
 done
}

bailout(){
 rm -f "${TMPFILES[@]}"
 exit $1
}

trap bailout 1 2 3 15

# Set braille device friendly screen size
if [ -z "$DISPLAY" ]; then
 TTY="$(tty 2>/dev/null)"
 STTYOPTS=""
 case "$TTY" in /dev/tty[0-9]*) CONSOLE="$TTY";; *) CONSOLE="";; esac
 # Set -ixon to avoid accidential console freezes by Control-S
 [ -n "$CONSOLE" ] && STTYOPTS="-ixon"
 if [ "$LINES" -gt "0" ] >/dev/null 2>&1; then
  export LINES
  [ -n "$CONSOLE" ] && STTYOPTS="$STTYOPTS rows $LINES"
 fi
 if [ "$COLUMNS" -gt "0" ] >/dev/null 2>&1; then
  export COLUMNS
   [ -n "$CONSOLE" ] && STTYOPTS="$STTYOPTS cols $COLUMNS"
 fi
 [ -n "$TERM" ] && export TERM
 [ -n "$STTYOPTS" ] && stty $STTYOPTS
else
 EDITOR="rxvt -fn 10x20 -e $EDITOR"
fi

[ -n "$EDITOR" ]      && export EDITOR
[ -n "$VIEWER" ]      && export VIEWER
[ -n "$LANG" ]        && export LANG
[ -n "$LANGUAGE" ]    && export LANGUAGE
[ -n "$LINGUAS" ]     && export LINGUAS
[ -n "$LC_LANG" ]     && export LC_LANG
[ -n "$LC_MESSAGES" ] && export LC_MESSAGES

# Common functions
say(){
 if [ -z "$DISPLAY" ]; then
  spd-say -P message "$1" >/dev/null 2>&1
 fi
}

# Produce a continuous number from anything that looks like a number.
numbertostderr(){
 echo "$1" 2>/dev/null | tr -d '[:alpha:][:blank:][:cntrl:][:punct:]' >&2
 return 0
}

# displayfile file title
# (Mis)use menu widget to display a file. --editbox does not do the same.
displayfile(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local text=()
 local count=0
 local lang="${LANG%%_*}"
 if [ -r "$1" ]; then
  while true; do
   text[$((count++))]=""
   read -r text[$((count++))] || break
  done < "$1"
 $DIALOG --title "$2" --no-cancel --visit-items --menu " " 23 "${DEFAULTWIDTH:-$COLUMNS}" 16 "${text[@]}" 2>/dev/null
 fi
 return 0
}

# Context-sensitive help (i.e. depends on how we were called)
# Unfinished; At least show the most important commands.
adriane_help(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local text=()
 local count=0
 local lang="${LANG%%_*}"
 case "$lang" in C) lang="en";; esac
 local module=""
 if [ -n "$ADRIANE_MODULE" ]; then
  module="-$ADRIANE_MODULE"
 fi

 local helpfile="/usr/share/doc/adriane-doc/help"$module"-"$lang".txt"
 if [ -r "$helpfile" ]; then
  displayfile "$helpfile" "$MSG_HELP"
 else
  error "$MSG_ERROR_HELP"
 fi
 return 0
}

# progress "Message" seconds_expected
progress(){
local TEXTDOMAINDIR="/usr/share/locale"
local TEXTDOMAIN="adriane-common"
if [ -n "$DISPLAY" ]; then
 if [ -n "$1" ]; then
  # Show progressbar
  touch "$TMP.progress"
  expected="$2"
  [ -n "$expected" ] && ratio="$((expected/10))+2" || ratio=4
  status=0
  ( while [ -e "$TMP.progress" ]; do echo "$(((status=(10000-status)/ratio+status)/100))"; sleep 1; done | $DIALOG --title "$1" --gauge "" 5 "${DEFAULTWIDTH:-75}" 0 ) &
 else
  # Kill previous
  rm -f "$TMP".progress ; wait
 fi
else
 [ -n "$1" ] && echo -n -e " $1\r" || echo -n -e "\r\033[K"
fi
 return 0
}

# non-blocking message
notify(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 $DIALOG --title "$TITLE" $NOOK --no-cancel --timeout 10 --visit-items --menu " " 0 "${DEFAULTWIDTH:-$COLUMNS}" 2 "" " $1 " 2>/dev/null &
 return 0
}

# yesno "text" [defaultno] returns 0 on yes, 1 on no.
yesno(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local RC=0
 local a=""
 if [ -n "$DISPLAY" ]; then
  $DIALOG --title "$TITLE" $NOOK --no-cancel --visit-items --menu " " 0 "$COLUMNS" 3 y "$1" n "$MSG_NO" 2>"$TMP".yesno || { rm -f "$TMP".yesno; return 1; }
  read a <"$TMP".yesno
  case "$a" in n) RC="1";; esac
  rm -f "$TMP".yesno
 else
  while true; do
   read -n 1 -s -r -p "$1" a
   if [ -z "$a" ]; then # empty answer, use default
    [ -n "$2" ] && RC="1"
    break
   fi
   case "$a" in [nN]|) RC="1"; break;;
               [jJyYsS]) RC="0"; break;;
   esac
  done
  [ "$RC" = "0" ] && echo -n -e "\n\r $MSG_YES" || echo -n -e "\n\r $MSG_NO"
  sleep 1
  echo ""
 fi
 return "$RC"
}

# prompt message that waits
prompt(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 $DIALOG --title "$TITLE" $NOOK --no-cancel --visit-items --menu " " 0 "${DEFAULTWIDTH:-$COLUMNS}" 2 "" " $1 " 2>/dev/null || return 1
 return 0
}

# error shortmessage [file contaning full message]
error(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local VIEWBUTTON=()
 $DIALOG --title "$MSG_ERROR" $NOOK --no-cancel --visit-items --menu " " 0 "${DEFAULTWIDTH:-$COLUMNS}" 2 "" " $1 " 2>/dev/null || return 0
 [ -s "$2" ] && displayfile "$2" "$MSG_ERROR" 2>/dev/null
 return 0
}

# Main Menu without ANY buttons
# mainmenu
mainmenu(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local SELECTEDITEM=()
 [ -n "$DEFAULTITEM" ] && SELECTEDITEM=( "--default-item" "$DEFAULTITEM" ) 
 $DIALOG --title "$TITLE" $NOOK --no-cancel --ok-label "$MSG_SELECT" --cancel-label "$MSG_QUIT" "${SELECTEDITEM[@]}" --visit-items --menu " " 0 "${DEFAULTWIDTH:-$COLUMNS}" 0 "$@" || return 1
 return 0
}

# submenu item1 desc1 item2 desc2 ...
submenu(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local SELECTEDITEM=()
 [ -n "$DEFAULTITEM" ] && SELECTEDITEM=( "--default-item" "$DEFAULTITEM" ) 
 $DIALOG --title "$TITLE" $NOOK --no-cancel --ok-label "$MSG_SELECT" --cancel-label "$MSG_BACK" "${SELECTEDITEM[@]}" --visit-items --menu " " 0 "${DEFAULTWIDTH:-$COLUMNS}" 0 "$@" || return 1
 return 0
}

# editmenu item1 desc1 item2 desc2 ...
editmenu(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local SELECTEDITEM=()
 [ -n "$DEFAULTITEM" ] && SELECTEDITEM=( "--default-item" "$DEFAULTITEM" ) 
 $DIALOG --title "$TITLE" --ok-label "$MSG_EDIT" --cancel-label "$MSG_BACK" "${SELECTEDITEM[@]}" --visit-items --menu "" 0 "${DEFAULTWIDTH:-$COLUMNS}" 0 "$@" || return 1
 return 0
}

# textfield [default]
textfield(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 $DIALOG --title "$TITLE" $NOOK --no-cancel --cancel-label "$MSG_BACK" --inputbox " " 9 "${DEFAULTWIDTH:-$COLUMNS}" "$1" || return 1
 return 0
}

# textinput var prompt-text
textinput(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 if [ -n "$DISPLAY" ]; then
  $DIALOG --title "$TITLE" $NOOK --no-cancel --cancel-label "$MSG_BACK" --inputbox "$2" 9 "${DEFAULTWIDTH:-$COLUMNS}" "$1" 2>"$TMP".textinput || { rm -f "$TMP".textinput; return 1; }
  read "$1" < "$TMP".textinput; rm -f "$TMP".textinput
 else
  read -e -p "$2" "$1"
 fi
 return 0
}

# selectfile variable [directory]
selectfile(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local SELECTEDITEM=()
 [ -n "$DEFAULTITEM" ] && SELECTEDITEM=( "--default-item" "$DEFAULTITEM" ) 
 local dir="$2"
 [ -d "$dir" ] || dir="."
 while true; do
  unset FILE
  local FILE=("none")
  local fcount="${#FILE[@]}"
  unset LIST
  local LIST=("m" "$MSG_MANAGE_FILES")
  local lcount="${#LIST[@]}"
  local file=""
  local SELECTION
  local shortname
  while read file; do
   [ "$file" = "$dir" ] && continue
   shortname="${file##*/}"
   case "$shortname" in ..) true;; .*) continue;; esac
   FILE[$fcount]="$file"
   shortname="${shortname%.*}"
   [ -d "$file" ] && shortname="$(printf "%70s [DIR]" $shortname)"
   LIST[$((lcount++))]="$((fcount++))"
   LIST[$((lcount++))]="$shortname"
  done <<.
$(find "$dir" -maxdepth 1 -type d 2>/dev/null | sort -r)
$(find "$dir" -maxdepth 1 -type f 2>/dev/null | sort -r)
.
  rm -f "$TMP"
  $DIALOG --title "$TITLE" --ok-label "$MSG_SELECT" --cancel-label "$MSG_CANCEL" "${SELECTEDITEM[@]}" --visit-items --menu "$MSG_DIRECTORY $dir" 0 "${DEFAULTWIDTH:-$COLUMNS}" 0 "${LIST[@]}" 2>"$TMP".tmp || return 1
  read -r SELECTION <"$TMP".tmp ; rm -f "$TMP".tmp
  case "$SELECTION" in
   [1-9]*)
     file="${FILE[$SELECTION]}"
     if [ -f "$file" ]; then
      read -r "$1" <<.
$file
.
      return 0
     fi
     if [ -d "$file" ]; then
      case "$file" in */..) dir="${dir%/*}";; *) dir="$file";; esac
     fi
     ;;
    m) ( cd "$dir"; mc )
     ;;
  esac
 done
 return 0
}

# converttotext inputfile outputfile
converttotext(){
 local ft=$(dd if="$i" bs=10k count=1 | awk '/^[Cc]ontent-[Tt]ype: /{print $2}')
 [ -n "$ft" ] || ft=$(dd if="$i" bs=10k count=1 | awk '/^ *<[Hh][Tt][Mm][Ll] */{print $0}')
 [ -n "$ft" ] || ft=$(file "$1")
 case "$ft" in
  *[Hh][Tt][Mm][Ll]*)
   elinks -force-html -dump-width "$((COLUMNS - 1))" -no-connect -no-numbering -no-references $ELINKS_OPTS -dump "$1" > "$2";;
  *[Pp][Dd][Ff]*) pdftotext -layout "$1" "$2" ;;
  *Microsoft\ Office*) catdoc "$1" > "$2" ;;
  *text*) fold -s -w "$((COLUMNS - 1))" "$1" > "$2" ;;
  *) cp "$1" "$2" ;;
 esac
 return "$?"
}

notyet(){
 error "$@"
 return 0
}
